import fs from 'fs/promises';
import path from 'path';
import lunr from 'lunr';
import { getRelativeUrl } from '../../utilities/files.js';
const map = [];
function stripWhitespace(string) {
    return string.replace(/\s+/g, ' ');
}
export default function (options) {
    const opts = {
        searchBoxContainer: '#search-box',
        ignore: () => false,
        getTitle: doc => doc.querySelector('title')?.textContent?.trim() ?? '',
        getDescription: doc => doc.querySelector('meta[name="description"]')?.getAttribute('content') ?? '',
        getHeadings: doc => [...doc.querySelectorAll('h2, h3, h4')].map(heading => heading.textContent?.trim() ?? ''),
        getContent: doc => doc.querySelector('body')?.textContent?.trim() || '',
        directoryName: 'search',
        ...options
    };
    let pagesToIndex = [];
    return {
        name: 'doxicity-search-plugin',
        scripts: async (config) => {
            const clientSource = `
        // Append the search dialog to the body
        const siteSearch = document.createElement('div');
        const scrollbarWidth = Math.abs(window.innerWidth - document.documentElement.clientWidth);

        siteSearch.classList.add('site-search');
        siteSearch.innerHTML = \`
          <div class="site-search__overlay"></div>
          <dialog id="site-search-dialog" class="site-search__dialog">
            <div class="site-search__content">
              <div class="site-search__header">
                <div id="site-search-combobox" class="site-search__input-wrapper">
                  <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-search" viewBox="0 0 16 16">
                    <path d="M11.742 10.344a6.5 6.5 0 1 0-1.397 1.398h-.001c.03.04.062.078.098.115l3.85 3.85a1 1 0 0 0 1.415-1.414l-3.85-3.85a1.007 1.007 0 0 0-.115-.1zM12 6.5a5.5 5.5 0 1 1-11 0 5.5 5.5 0 0 1 11 0z"></path>
                  </svg>
                  <input
                    id="site-search-input"
                    class="site-search__input"
                    type="search"
                    placeholder="Search"
                    autocomplete="off"
                    autocorrect="off"
                    autocapitalize="off"
                    enterkeyhint="go"
                    spellcheck="false"
                    maxlength="100"
                    role="combobox"
                    aria-autocomplete="list"
                    aria-expanded="true"
                    aria-controls="site-search-listbox"
                    aria-haspopup="listbox"
                    aria-activedescendant
                  >
                  <button type="button" class="site-search__clear-button" aria-label="Clear entry" tabindex="-1" hidden>
                    <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-x-circle-fill" viewBox="0 0 16 16">
                      <path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM5.354 4.646a.5.5 0 1 0-.708.708L7.293 8l-2.647 2.646a.5.5 0 0 0 .708.708L8 8.707l2.646 2.647a.5.5 0 0 0 .708-.708L8.707 8l2.647-2.646a.5.5 0 0 0-.708-.708L8 7.293 5.354 4.646z"></path>
                    </svg>
                  </button>
                </div>
              </div>
              <div class="site-search__body">
                <ul
                  id="site-search-listbox"
                  class="site-search__results"
                  role="listbox"
                  aria-label="Search results"
                ></ul>
                <div class="site-search__empty">No matching pages</div>
              </div>
              <footer class="site-search__footer">
                <small><kbd>↑</kbd> <kbd>↓</kbd> Navigate</small>
                <small><kbd>↲</kbd> Select</small>
                <small><kbd>Esc</kbd> Close</small>
              </footer>
            </div>
          </dialog>
        \`;

        const overlay = siteSearch.querySelector('.site-search__overlay');
        const dialog = siteSearch.querySelector('.site-search__dialog');
        const input = siteSearch.querySelector('.site-search__input');
        const clearButton = siteSearch.querySelector('.site-search__clear-button');
        const results = siteSearch.querySelector('.site-search__results');
        const animationDuration = 150;
        const searchDebounce = 100;
        let isShowing = false;
        let searchTimeout;
        let searchIndex;
        let map;

        // Load search data
        const searchIndexFetch = fetch('SEARCH_INDEX_URL')
          .then(res => res.json())
          .then(data => {
            // eslint-disable-next-line no-undef
            searchIndex = lunr.Index.load(data.searchIndex);
            map = data.map;
          });

        async function show() {
          isShowing = true;
          document.body.append(siteSearch);
          document.body.classList.add('site-search-visible');
          document.body.style.setProperty('--dox-search-scroll-lock-size', \`\${scrollbarWidth}px\`);
          clearButton.hidden = true;
          requestAnimationFrame(() => input.focus());
          updateResults();

          dialog.showModal();

          await Promise.all([
            dialog.animate(
              [
                { opacity: 0, transform: 'scale(.9)', transformOrigin: 'top' },
                { opacity: 1, transform: 'scale(1)', transformOrigin: 'top' }
              ],
              { duration: animationDuration }
            ).finished,
            overlay.animate([{ opacity: 0 }, { opacity: 1 }], { duration: animationDuration }).finished
          ]);

          dialog.addEventListener('mousedown', handleMouseDown);
          dialog.addEventListener('keydown', handleKeyDown);
        }

        async function hide() {
          isShowing = false;

          await Promise.all([
            dialog.animate(
              [
                { opacity: 1, transform: 'scale(1)', transformOrigin: 'top' },
                { opacity: 0, transform: 'scale(.9)', transformOrigin: 'top' }
              ],
              { duration: animationDuration }
            ).finished,
            overlay.animate([{ opacity: 1 }, { opacity: 0 }], { duration: animationDuration }).finished
          ]);

          dialog.close();

          input.blur(); // otherwise Safari will scroll to the bottom of the page on close
          input.value = '';
          document.body.classList.remove('site-search-visible');
          document.body.style.removeProperty('--dox-search-scroll-lock-size');
          siteSearch.remove();
          updateResults();

          dialog.removeEventListener('mousedown', handleMouseDown);
          dialog.removeEventListener('keydown', handleKeyDown);
        }

        function handleInput() {
          clearButton.hidden = input.value === '';

          // Debounce search queries
          clearTimeout(searchTimeout);
          searchTimeout = setTimeout(() => updateResults(input.value), searchDebounce);
        }

        function handleClear() {
          clearButton.hidden = true;
          input.value = '';
          input.focus();
          updateResults();
        }

        function handleMouseDown(event) {
          if (!event.target.closest('.site-search__content')) {
            hide();
          }
        }

        function handleKeyDown(event) {
          // Close when pressing escape
          if (event.key === 'Escape') {
            event.preventDefault(); // prevent <dialog> from closing immediately so it can animate
            event.stopImmediatePropagation();
            hide();
            return;
          }

          // Handle keyboard selections
          if (['ArrowDown', 'ArrowUp', 'Home', 'End', 'Enter'].includes(event.key)) {
            event.preventDefault();

            const currentEl = results.querySelector('[data-selected="true"]');
            const items = [...results.querySelectorAll('li')];
            const index = items.indexOf(currentEl);
            let nextEl;

            if (items.length === 0) {
              return;
            }

            switch (event.key) {
              case 'ArrowUp':
                nextEl = items[Math.max(0, index - 1)];
                break;
              case 'ArrowDown':
                nextEl = items[Math.min(items.length - 1, index + 1)];
                break;
              case 'Home':
                nextEl = items[0];
                break;
              case 'End':
                nextEl = items[items.length - 1];
                break;
              case 'Enter':
                currentEl?.querySelector('a')?.click();
                break;
            }

            // Update the selected item
            items.forEach(item => {
              if (item === nextEl) {
                input.setAttribute('aria-activedescendant', item.id);
                item.setAttribute('data-selected', 'true');
                nextEl.scrollIntoView({ block: 'nearest' });
              } else {
                item.setAttribute('data-selected', 'false');
              }
            });
          }
        }

        async function updateResults(query = '') {
          try {
            await searchIndexFetch;

            const hasQuery = query.length > 0;
            const searchTokens = query
              .split(' ')
              .map((term, index, arr) => \`\${term}\${index === arr.length - 1 ? \`* \${term}~1\` : '~1'}\`)
              .join(' ');
            const matches = hasQuery ? searchIndex.search(\`\${query} \${searchTokens}\`) : [];
            const hasResults = hasQuery && matches.length > 0;

            siteSearch.classList.toggle('site-search--has-results', hasQuery && hasResults);
            siteSearch.classList.toggle('site-search--no-results', hasQuery && !hasResults);

            input.setAttribute('aria-activedescendant', '');
            results.innerHTML = '';

            matches.forEach((match, index) => {
              const page = map[match.ref];
              const li = document.createElement('li');
              const a = document.createElement('a');
              const displayTitle = page.title ?? '';
              const displayDescription = page.description ?? '';
              const displayUrl = page.url.replace(/^\\//, '');

              li.classList.add('site-search__result');
              li.setAttribute('role', 'option');
              li.setAttribute('id', \`search-result-item-\${match.ref}\`);
              li.setAttribute('data-selected', index === 0 ? 'true' : 'false');

              a.href = page.url;
              a.innerHTML = \`
                <div class="site-search__result-icon" aria-hidden="true">
                  <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-file-text" viewBox="0 0 16 16">
                    <path d="M5 4a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1H5zm-.5 2.5A.5.5 0 0 1 5 6h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1-.5-.5zM5 8a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1H5zm0 2a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1H5z"></path>
                    <path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"></path>
                  </svg>
                </div>
                <div class="site-search__result__details">
                  <div class="site-search__result-title"></div>
                  <div class="site-search__result-description"></div>
                  <div class="site-search__result-url"></div>
                </div>
              \`;
              a.querySelector('.site-search__result-title').textContent = displayTitle;
              a.querySelector('.site-search__result-description').textContent = displayDescription;
              a.querySelector('.site-search__result-url').textContent = displayUrl;

              li.appendChild(a);
              results.appendChild(li);
            });
          } catch {
            // Ignore query errors as the user types
          }
        }

        // Show the search dialog when clicking on data-plugin="search"
        document.addEventListener('click', event => {
          const searchButton = event.target.closest('[data-plugin="search"]');
          if (searchButton) {
            show();
          }
        });

        // Show the search dialog when slash (or CMD+K) is pressed and focus is not inside a form element
        document.addEventListener('keydown', event => {
          if (
            !isShowing &&
            (event.key === '/' || (event.key === 'k' && (event.metaKey || event.ctrlKey))) &&
            !event.composedPath().some(el => ['input', 'textarea'].includes(el?.tagName?.toLowerCase()))
          ) {
            event.preventDefault();
            show();
          }
        });

        input.addEventListener('input', handleInput);
        clearButton.addEventListener('click', handleClear);

        // Close when a result is selected
        results.addEventListener('click', event => {
          if (event.target.closest('a')) {
            hide();
          }
        });
      `;
            const lunrSource = await fs.readFile('./node_modules/lunr/lunr.min.js', 'utf8');
            const searchIndexFilename = path.join(config.destination, config.assets, path.join(opts.directoryName, 'search-index.json'));
            const searchIndexUrl = getRelativeUrl(config, searchIndexFilename);
            const scripts = `
        /* prettier-ignore */
        ${lunrSource}

        ${clientSource.replace(/SEARCH_INDEX_URL/g, searchIndexUrl)}
      `;
            return scripts;
        },
        styles: () => {
            return `
        :root {
          --dox-search-box-background: white;
          --dox-search-box-border-width: 1px;
          --dox-search-box-border-color: #ddd;
          --dox-search-box-color: #232323;
          --dox-search-dialog-background: white;
          --dox-search-border-width: 1px;
          --dox-search-border-color: #ddd;
          --dox-search-text-color: #232323;
          --dox-search-text-color-muted: #888;
          --dox-search-font-weight-normal: normal;
          --dox-search-font-weight-semibold: 500;
          --dox-search-border-radius: 8px;
          --dox-search-accent-color: green;
          --dox-search-icon-color: #888;
          --dox-search-icon-color-active: #777;
          --dox-search-shadow: 0 4px 16px hsl(240 3.8% 46.1% / 24%);
          --dox-search-result-background-hover: #eee;
          --dox-search-result-color-hover: #232323;
          --dox-search-result-background-active: var(--dox-search-accent-color);
          --dox-search-result-color-active: white;
          --dox-search-focus-ring: solid 3px green;
          --dox-search-overlay-background: rgba(0 0 0 / 0.5);
        }

        body.site-search-visible {
          padding-right: var(--dox-search-scroll-lock-size) !important;
          overflow: hidden !important;
        }

        /* Search box */
        .site-search-box {
          flex: 1 1 auto;
          display: flex;
          align-items: center;
          width: 100%;
          border: none;
          border-radius: 9999px;
          background: var(--dox-search-box-background);
          border: solid var(--dox-search-box-border-width) var(--dox-search-box-border-color);
          font: inherit;
          color: var(--dox-search-box-color);
          padding: 0.75rem 1rem;
          margin: 0;
          cursor: pointer;
        }

        .site-search-box span {
          flex: 1 1 auto;
          width: 1rem;
          height: 1rem;
          text-align: left;
          line-height: 1;
          margin: 0 1rem;
        }

        .site-search-box:focus {
          outline: none;
        }

        .site-search-box:focus-visible {
          outline: var(--dox-search-focus-ring);
        }

        /* Site search */
        .site-search {
          position: fixed;
          top: 0;
          left: 0;
          right: 0;
          bottom: 0;
          z-index: 9999;
        }

        .site-search[hidden] {
          display: none;
        }

        .site-search__overlay {
          position: absolute;
          top: 0;
          left: 0;
          right: 0;
          bottom: 0;
          background: var(--dox-search-overlay-background);
          z-index: -1;
        }

        .site-search__dialog {
          width: 100%;
          height: 100%;
          max-width: none;
          max-height: none;
          background: transparent;
          border: none;
          padding: 0;
          margin: 0;
        }

        .site-search__dialog:focus {
          outline: none;
        }

        .site-search__dialog::backdrop {
          display: none;
        }

        /* Fixes an iOS Safari 16.4 bug that draws the parent element's border radius incorrectly when showing/hiding results */
        .site-search__header {
          background-color: var(--dox-search-dialog-background);
          border-radius: var(--dox-search-border-radius);
        }

        .site-search--has-results .site-search__header {
          border-top-left-radius: var(--dox-search-border-radius);
          border-top-right-radius: var(--dox-search-border-radius);
          border-bottom-left-radius: 0;
          border-bottom-right-radius: 0;
        }

        .site-search__content {
          display: flex;
          flex-direction: column;
          width: 100%;
          max-width: 500px;
          max-height: calc(100vh - 20rem);
          background-color: var(--dox-search-dialog-background);
          border-radius: var(--dox-search-border-radius);
          box-shadow: var(--dox-search-shadow);
          padding: 0;
          margin: 10rem auto;
        }

        @media screen and (max-width: 900px) {
          .site-search__content {
            max-width: calc(100% - 2rem);
            max-height: calc(90svh);
            margin: 4vh 1rem;
          }
        }

        .site-search__input-wrapper {
          display: flex;
          align-items: center;
        }

        .site-search__input-wrapper svg {
          width: 1.5rem;
          height: 1.5rem;
          flex: 0 0 auto;
          color: var(--dox-search-icon-color);
          margin: 0 1.5rem;
        }

        .site-search__clear-button {
          display: flex;
          background: none;
          border: none;
          font: inherit;
          padding: 0;
          margin: 0;
          cursor: pointer;
        }

        .site-search__clear-button[hidden] {
          display: none;
        }

        .site-search__clear-button:active svg {
          color: var(--dox-search-icon-color-active);
        }

        .site-search__input {
          flex: 1 1 auto;
          min-width: 0;
          border: none;
          font: inherit;
          font-size: 1.5rem;
          font-weight: var(--dox-search-font-weight-normal);
          color: var(--dox-search-text-color);
          background: transparent;
          padding: 1rem 0;
          margin: 0;
        }

        .site-search__input::placeholder {
          color: var(--dox-search-text-color-muted);
        }

        .site-search__input::-webkit-search-decoration,
        .site-search__input::-webkit-search-cancel-button,
        .site-search__input::-webkit-search-results-button,
        .site-search__input::-webkit-search-results-decoration {
          display: none;
        }

        .site-search__input:focus,
        .site-search__input:focus-visible {
          outline: none;
        }

        .site-search__body {
          flex: 1 1 auto;
          overflow: auto;
        }

        .site-search--has-results .site-search__body {
          border-top: solid var(--dox-search-border-width) var(--dox-search-border-color);
        }

        .site-search__results {
          display: none;
          line-height: 1.2;
          list-style: none;
          padding: 0.5rem 0;
          margin: 0;
        }

        .site-search--has-results .site-search__results {
          display: block;
        }

        .site-search__results a {
          display: block;
          text-decoration: none;
          padding: 0.5rem 1.5rem;
        }

        .site-search__results a:focus-visible {
          outline: var(--dox-search-focus-ring);
        }

        .site-search__results li a:hover,
        .site-search__results li a:hover small {
          background-color: var(--dox-search-result-background-hover);
          color: var(--dox-search-result-color-hover);
        }

        .site-search__results li[data-selected='true'] a,
        .site-search__results li[data-selected='true'] a * {
          outline: none;
          background-color: var(--dox-search-result-background-active);
          color: var(--dox-search-result-color-active);
        }

        .site-search__results h3 {
          font-weight: var(--dox-search-font-weight-semibold);
          margin: 0;
        }

        .site-search__results small {
          display: block;
          color: var(--dox-search-text-color-muted);
        }

        .site-search__result {
          padding: 0;
          margin: 0;
        }

        .site-search__result a {
          display: flex;
          align-items: center;
          gap: 1rem;
        }

        .site-search__result-icon {
          flex: 0 0 auto;
          display: flex;
          color: var(--dox-search-text-color-muted);
        }

        .site-search__result-icon svg {
          width: 1.5rem;
          height: 1.5rem;
        }

        .site-search__result__details {
          width: calc(100% - 3rem);
        }

        .site-search__result-title,
        .site-search__result-description,
        .site-search__result-url {
          max-width: 400px;
          line-height: 1.3;
          white-space: nowrap;
          overflow: hidden;
          text-overflow: ellipsis;
        }

        .site-search__result-title {
          font-size: 1.2rem;
          font-weight: var(--dox-search-font-weight-semibold);
          color: var(--dox-search-accent-color);
        }

        .site-search__result-description {
          font-size: 0.875rem;
          color: var(--dox-search-text-color);
        }

        .site-search__result-url {
          font-size: 0.875rem;
          color: var(--dox-search-text-color-muted);
        }

        .site-search__empty {
          display: none;
          border-top: solid var(--dox-search-border-width) var(--dox-search-border-color);
          text-align: center;
          color: var(--dox-search-text-color-muted);
          padding: 2rem;
        }

        .site-search--no-results .site-search__empty {
          display: block;
        }

        .site-search__footer {
          display: flex;
          justify-content: center;
          gap: 2rem;
          border-top: solid var(--dox-search-border-width) var(--dox-search-border-color);
          border-bottom-left-radius: inherit;
          border-bottom-right-radius: inherit;
          padding: 1rem;
        }

        .site-search__footer small {
          color: var(--dox-search-text-color-muted);
        }

        .site-search__footer small kbd:last-of-type {
          margin-right: 0.25rem;
        }

        @media screen and (max-width: 900px) {
          .site-search__footer {
            display: none;
          }
        }
      `;
        },
        transform: (doc, page, config) => {
            const searchContainer = doc.querySelector(opts.searchBoxContainer);
            const searchBox = doc.createElement('button');
            searchBox.classList.add('site-search-box');
            searchBox.type = 'button';
            searchBox.title = 'Press / to search';
            searchBox.setAttribute('aria-label', 'Search');
            searchBox.setAttribute('data-plugin', 'search');
            searchBox.innerHTML = `
        <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" viewBox="0 0 16 16" aria-hidden="true">
          <path d="M11.742 10.344a6.5 6.5 0 1 0-1.397 1.398h-.001c.03.04.062.078.098.115l3.85 3.85a1 1 0 0 0 1.415-1.414l-3.85-3.85a1.007 1.007 0 0 0-.115-.1zM12 6.5a5.5 5.5 0 1 1-11 0 5.5 5.5 0 0 1 11 0z"></path>
        </svg>
        <span>Search</span>
      `;
            searchContainer?.append(searchBox);
            const title = stripWhitespace(opts.getTitle(doc));
            const description = stripWhitespace(opts.getDescription(doc));
            const headings = opts.getHeadings(doc).map(stripWhitespace);
            const content = stripWhitespace(opts.getContent(doc));
            const url = getRelativeUrl(config, page.destinationFile);
            pagesToIndex.push({ page, title, description, headings, content, url });
            return doc;
        },
        afterAll: (_pages, config) => {
            const searchIndexFilename = path.join(config.destination, config.assets, path.join(opts.directoryName, 'search-index.json'));
            const searchIndex = lunr(async function () {
                let index = 0;
                this.ref('id');
                this.field('t', { boost: 10 });
                this.field('h', { boost: 5 });
                this.field('c');
                for (const page of pagesToIndex) {
                    if (opts.ignore(page.page))
                        continue;
                    this.add({ id: index, t: page.title, h: page.headings, c: page.content });
                    map[index] = { title: page.title, description: page.description, url: page.url };
                    index++;
                }
                await fs.mkdir(path.dirname(searchIndexFilename), { recursive: true });
                await fs.writeFile(searchIndexFilename, JSON.stringify({ searchIndex, map }), 'utf8');
                pagesToIndex = [];
            });
        }
    };
}
